﻿using System;

namespace howto_escape_time_polynomial
{
    public class Complex
    {
        public double Re { get; set; }
        public double Im { get; set; }

        public Complex(double re, double im)
        {
            Re = re;
            Im = im;
        }
        public Complex(double re)
            : this(re, 0)
        {
        }
        public Complex()
            : this(0, 0)
        {
        }

        // Magnitude.
        public double Magnitude
        {
            get
            {
                return Math.Sqrt(Re * Re + Im * Im);
            }
        }

        // Square root.
        public Complex Sqrt
        {
            get
            {
                double new_re = Math.Sqrt((Magnitude + Re) / 2.0);
                double new_im = Math.Sign(Im) * Math.Sqrt((Magnitude - Re) / 2.0);
                return new Complex(new_re, new_im);
            }
        }

        // Return the polar version of the number.
        public void ToPolar(out double r, out double theta)
        {
            r = Math.Sqrt(Re * Re + Im * Im);
            theta = Math.Atan2(Im, Re);
        }

        // Use De Moivre’s equation to raise the number to a power.
        public Complex Pow(double power)
        {
            // Get the polar representation.
            double r, theta;
            ToPolar(out r, out theta);

            // Use De Moivre’s equation.
            double r_to_power = Math.Pow(r, power);
            double new_re = r_to_power * Math.Cos(power * theta);
            double new_im = r_to_power * Math.Sin(power * theta);
            return new Complex(new_re, new_im);
        }

        // Multiplication.
        public static Complex operator *(Complex c1, Complex c2)
        {
            return new Complex(
                c1.Re * c2.Re - c1.Im * c2.Im,
                c1.Re * c2.Im + c1.Im * c2.Re);
        }

        // Return 1/c.
        public Complex Inverse()
        {
            double magnitude = Magnitude;
            return new Complex(Re / magnitude, -Im / magnitude);
        }

        // Division.
        public static Complex operator /(Complex c1, Complex c2)
        {
            return c1 * c2.Inverse();
        }

        // Unary negation.
        public static Complex operator -(Complex c1)
        {
            return new Complex(-c1.Re, -c1.Im);
        }

        // Addition.
        public static Complex operator +(Complex c1, Complex c2)
        {
            return new Complex(c1.Re + c2.Re, c1.Im + c2.Im);
        }

        // Subtraction.
        public static Complex operator -(Complex c1, Complex c2)
        {
            return c1 + (-c2);
        }

        // Convert double to Complex.
        public static implicit operator Complex(double d)
        {
            return new Complex(d);
        }

        public override string ToString()
        {
            return string.Format("{0} + {1}i", Re, Im);
        }

        // Parse a String of the form R + Ii.
        public static Complex Parse(string string_value)
        {
            // Remove spaces and the "i".
            string_value = string_value.Replace(" ", "");
            string_value = string_value.ToLower().Replace("i", "");

            // Find the + or - between the parts.
            char[] plus_or_minus = { '+', '-' };
            int pos = string_value.IndexOfAny(plus_or_minus);
            if (pos == 0)
            {
                // Skip the leading +/-.
                pos = string_value.IndexOfAny(plus_or_minus, 1);
            }

            // Get the real and imaginary parts.
            double re = double.Parse(string_value.Substring(0, pos));
            double im = double.Parse(string_value.Substring(pos));

            return new Complex(re, im);
        }
    }
}
